@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2016 Contributors as noted in the AUTHORS file
 
     This file is part of libzmq, the ZeroMQ core engine in C++.
 
@@ -27,8 +27,8 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "precompiled.hpp"
 #include "clock.hpp"
-#include "platform.hpp"
 #include "likely.hpp"
 #include "config.hpp"
 #include "err.hpp"
@@ -52,46 +52,85 @@
 #include <time.h>
 #endif
 
+#if defined ZMQ_HAVE_VXWORKS
+#include "timers.h"
+#endif
+
+#if defined ZMQ_HAVE_OSX
+#include <mach/clock.h>
+#include <mach/mach.h>
+#include <time.h>
+#include <sys/time.h>
+
+int alt_clock_gettime (int clock_id, timespec *ts)
+{
+    clock_serv_t cclock;
+    mach_timespec_t mts;
+    host_get_clock_service (mach_host_self (), clock_id, &cclock);
+    clock_get_time (cclock, &mts);
+    mach_port_deallocate (mach_task_self (), cclock);
+    ts->tv_sec = mts.tv_sec;
+    ts->tv_nsec = mts.tv_nsec;
+    return 0;
+}
+#endif
+
 #ifdef ZMQ_HAVE_WINDOWS
-typedef ULONGLONG (*f_compatible_get_tick_count64)();
+typedef ULONGLONG (*f_compatible_get_tick_count64) ();
 
 static zmq::mutex_t compatible_get_tick_count64_mutex;
 
-ULONGLONG compatible_get_tick_count64()
+ULONGLONG compatible_get_tick_count64 ()
 {
-  compatible_get_tick_count64_mutex.lock();
-  static DWORD s_wrap = 0;
-  static DWORD s_last_tick = 0;
-  const DWORD current_tick = ::GetTickCount();
-  if (current_tick < s_last_tick)
-    ++s_wrap;
+#ifdef ZMQ_HAVE_WINDOWS_UWP
+    const ULONGLONG result = ::GetTickCount64 ();
+    return result;
+#else
+    zmq::scoped_lock_t locker (compatible_get_tick_count64_mutex);
 
-  s_last_tick = current_tick;
-  const ULONGLONG result = (static_cast<ULONGLONG>(s_wrap) << 32) + static_cast<ULONGLONG>(current_tick);
-  compatible_get_tick_count64_mutex.unlock();
-  return result;
+    static DWORD s_wrap = 0;
+    static DWORD s_last_tick = 0;
+    const DWORD current_tick = ::GetTickCount ();
+
+    if (current_tick < s_last_tick)
+        ++s_wrap;
+
+    s_last_tick = current_tick;
+    const ULONGLONG result = (static_cast<ULONGLONG> (s_wrap) << 32)
+                             + static_cast<ULONGLONG> (current_tick);
+
+    return result;
+#endif
 }
 
-f_compatible_get_tick_count64 init_compatible_get_tick_count64()
+f_compatible_get_tick_count64 init_compatible_get_tick_count64 ()
 {
-  f_compatible_get_tick_count64 func = NULL;
-  HMODULE module = ::LoadLibraryA("Kernel32.dll");
-  if (module != NULL)
-    func = reinterpret_cast<f_compatible_get_tick_count64>(::GetProcAddress(module, "GetTickCount64"));
+    f_compatible_get_tick_count64 func = NULL;
+#if !defined ZMQ_HAVE_WINDOWS_UWP
 
-  if (func == NULL)
-    func = compatible_get_tick_count64;
+    HMODULE module = ::LoadLibraryA ("Kernel32.dll");
+    if (module != NULL)
+        func = reinterpret_cast<f_compatible_get_tick_count64> (
+          ::GetProcAddress (module, "GetTickCount64"));
+#endif
+    if (func == NULL)
+        func = compatible_get_tick_count64;
 
-  return func;
+#if !defined ZMQ_HAVE_WINDOWS_UWP
+    ::FreeLibrary (module);
+#endif
+
+    return func;
 }
 
-static f_compatible_get_tick_count64 my_get_tick_count64 = init_compatible_get_tick_count64();
+static f_compatible_get_tick_count64 my_get_tick_count64 =
+  init_compatible_get_tick_count64 ();
 #endif
 
 zmq::clock_t::clock_t () :
     last_tsc (rdtsc ()),
 #ifdef ZMQ_HAVE_WINDOWS
-    last_time (static_cast<uint64_t>((*my_get_tick_count64)()))
+    last_time (static_cast<uint64_t> ((*my_get_tick_count64) ()))
 #else
     last_time (now_us () / 1000)
 #endif
@@ -119,21 +158,30 @@ uint64_t zmq::clock_t::now_us ()
     double ticks_div = ticksPerSecond.QuadPart / 1000000.0;
     return (uint64_t) (tick.QuadPart / ticks_div);
 
-#elif defined HAVE_CLOCK_GETTIME && defined CLOCK_MONOTONIC
+#elif defined HAVE_CLOCK_GETTIME                                               \
+  && (defined CLOCK_MONOTONIC || defined ZMQ_HAVE_VXWORKS)
 
     //  Use POSIX clock_gettime function to get precise monotonic time.
     struct timespec tv;
+
+#if defined ZMQ_HAVE_OSX                                                       \
+  && __MAC_OS_X_VERSION_MIN_REQUIRED < 101200 // less than macOS 10.12
+    int rc = alt_clock_gettime (SYSTEM_CLOCK, &tv);
+#else
     int rc = clock_gettime (CLOCK_MONOTONIC, &tv);
-		// Fix case where system has clock_gettime but CLOCK_MONOTONIC is not supported.
-		// This should be a configuration check, but I looked into it and writing an
-		// AC_FUNC_CLOCK_MONOTONIC seems beyond my powers.
-		if( rc != 0) {
-			//  Use POSIX gettimeofday function to get precise time.
-			struct timeval tv;
-			int rc = gettimeofday (&tv, NULL);
-			errno_assert (rc == 0);
-			return (tv.tv_sec * (uint64_t) 1000000 + tv.tv_usec);
-		}
+#endif
+    // Fix case where system has clock_gettime but CLOCK_MONOTONIC is not supported.
+    // This should be a configuration check, but I looked into it and writing an
+    // AC_FUNC_CLOCK_MONOTONIC seems beyond my powers.
+    if (rc != 0) {
+#ifndef ZMQ_HAVE_VXWORKS
+        //  Use POSIX gettimeofday function to get precise time.
+        struct timeval tv;
+        int rc = gettimeofday (&tv, NULL);
+        errno_assert (rc == 0);
+        return (tv.tv_sec * (uint64_t) 1000000 + tv.tv_usec);
+#endif
+    }
     return (tv.tv_sec * (uint64_t) 1000000 + tv.tv_nsec / 1000);
 
 #elif defined HAVE_GETHRTIME
@@ -156,14 +204,13 @@ uint64_t zmq::clock_t::now_ms ()
     uint64_t tsc = rdtsc ();
 
     //  If TSC is not supported, get precise time and chop off the microseconds.
-    if (!tsc)
-    {
+    if (!tsc) {
 #ifdef ZMQ_HAVE_WINDOWS
         // Under Windows, now_us is not so reliable since QueryPerformanceCounter
         // does not guarantee that it will use a hardware that offers a monotonic timer.
         // So, lets use GetTickCount when GetTickCount64 is not available with an workaround
         // to its 32 bit limitation.
-        return static_cast<uint64_t>((*my_get_tick_count64)());
+        return static_cast<uint64_t> ((*my_get_tick_count64) ());
 #else
         return now_us () / 1000;
 #endif
@@ -177,7 +224,7 @@ uint64_t zmq::clock_t::now_ms ()
 
     last_tsc = tsc;
 #ifdef ZMQ_HAVE_WINDOWS
-    last_time = static_cast<uint64_t>((*my_get_tick_count64)());
+    last_time = static_cast<uint64_t> ((*my_get_tick_count64) ());
 #else
     last_time = now_us () / 1000;
 #endif
@@ -190,21 +237,29 @@ uint64_t zmq::clock_t::rdtsc ()
     return __rdtsc ();
 #elif (defined __GNUC__ && (defined __i386__ || defined __x86_64__))
     uint32_t low, high;
-    __asm__ volatile ("rdtsc" : "=a" (low), "=d" (high));
+    __asm__ volatile("rdtsc" : "=a"(low), "=d"(high));
     return (uint64_t) high << 32 | low;
-#elif (defined __SUNPRO_CC && (__SUNPRO_CC >= 0x5100) && (defined __i386 || \
-    defined __amd64 || defined __x86_64))
-    union {
+#elif (defined __SUNPRO_CC && (__SUNPRO_CC >= 0x5100)                          \
+       && (defined __i386 || defined __amd64 || defined __x86_64))
+    union
+    {
         uint64_t u64val;
-        uint32_t u32val [2];
+        uint32_t u32val[2];
     } tsc;
-    asm("rdtsc" : "=a" (tsc.u32val [0]), "=d" (tsc.u32val [1]));
+    asm("rdtsc" : "=a"(tsc.u32val[0]), "=d"(tsc.u32val[1]));
     return tsc.u64val;
 #elif defined(__s390__)
     uint64_t tsc;
-    asm("\tstck\t%0\n" : "=Q" (tsc) : : "cc");
-    return(tsc);
+    asm("\tstck\t%0\n" : "=Q"(tsc) : : "cc");
+    return (tsc);
 #else
-    return 0;
+    struct timespec ts;
+#if defined ZMQ_HAVE_OSX                                                       \
+  && __MAC_OS_X_VERSION_MIN_REQUIRED < 101200 // less than macOS 10.12
+    alt_clock_gettime (SYSTEM_CLOCK, &ts);
+#else
+    clock_gettime (CLOCK_MONOTONIC, &ts);
+#endif
+    return (uint64_t) (ts.tv_sec) * 1000000000 + ts.tv_nsec;
 #endif
 }
