@@ -1,22 +1,33 @@
 /*
-    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2016 Contributors as noted in the AUTHORS file
 
-    This file is part of 0MQ.
+    This file is part of libzmq, the ZeroMQ core engine in C++.
 
-    0MQ is free software; you can redistribute it and/or modify it under
-    the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 3 of the License, or
+    libzmq is free software; you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
-    0MQ is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
+    As a special exception, the Contributors give you permission to link
+    this library with independent modules to produce an executable,
+    regardless of the license terms of these independent modules, and to
+    copy and distribute the resulting executable under terms of your choice,
+    provided that you also meet, for each linked independent module, the
+    terms and conditions of the license of that module. An independent
+    module is a module which is not derived from or based on this library.
+    If you modify this library, you must extend this exception to your
+    version of the library.
+
+    libzmq is distributed in the hope that it will be useful, but WITHOUT
+    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
+    License for more details.
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "precompiled.hpp"
 #include "kqueue.hpp"
 #if defined ZMQ_USE_KQUEUE
 
@@ -28,6 +39,7 @@
 #include <algorithm>
 #include <new>
 
+#include "macros.hpp"
 #include "kqueue.hpp"
 #include "err.hpp"
 #include "config.hpp"
@@ -42,28 +54,29 @@
 #define kevent_udata_t void *
 #endif
 
-zmq::kqueue_t::kqueue_t () :
-    stopping (false)
+zmq::kqueue_t::kqueue_t (const zmq::thread_ctx_t &ctx_) :
+    worker_poller_base_t (ctx_)
 {
     //  Create event queue
     kqueue_fd = kqueue ();
     errno_assert (kqueue_fd != -1);
 #ifdef HAVE_FORK
-    pid = getpid();
+    pid = getpid ();
 #endif
 }
 
 zmq::kqueue_t::~kqueue_t ()
 {
-    worker.stop ();
+    stop_worker ();
     close (kqueue_fd);
 }
 
 void zmq::kqueue_t::kevent_add (fd_t fd_, short filter_, void *udata_)
 {
+    check_thread ();
     struct kevent ev;
 
-    EV_SET (&ev, fd_, filter_, EV_ADD, 0, 0, (kevent_udata_t)udata_);
+    EV_SET (&ev, fd_, filter_, EV_ADD, 0, 0, (kevent_udata_t) udata_);
     int rc = kevent (kqueue_fd, &ev, 1, NULL, 0, NULL);
     errno_assert (rc != -1);
 }
@@ -78,8 +91,9 @@ void zmq::kqueue_t::kevent_delete (fd_t fd_, short filter_)
 }
 
 zmq::kqueue_t::handle_t zmq::kqueue_t::add_fd (fd_t fd_,
-    i_poll_events *reactor_)
+                                               i_poll_events *reactor_)
 {
+    check_thread ();
     poll_entry_t *pe = new (std::nothrow) poll_entry_t;
     alloc_assert (pe);
 
@@ -95,7 +109,8 @@ zmq::kqueue_t::handle_t zmq::kqueue_t::add_fd (fd_t fd_,
 
 void zmq::kqueue_t::rm_fd (handle_t handle_)
 {
-    poll_entry_t *pe = (poll_entry_t*) handle_;
+    check_thread ();
+    poll_entry_t *pe = (poll_entry_t *) handle_;
     if (pe->flag_pollin)
         kevent_delete (pe->fd, EVFILT_READ);
     if (pe->flag_pollout)
@@ -108,7 +123,8 @@ void zmq::kqueue_t::rm_fd (handle_t handle_)
 
 void zmq::kqueue_t::set_pollin (handle_t handle_)
 {
-    poll_entry_t *pe = (poll_entry_t*) handle_;
+    check_thread ();
+    poll_entry_t *pe = (poll_entry_t *) handle_;
     if (likely (!pe->flag_pollin)) {
         pe->flag_pollin = true;
         kevent_add (pe->fd, EVFILT_READ, pe);
@@ -117,7 +133,8 @@ void zmq::kqueue_t::set_pollin (handle_t handle_)
 
 void zmq::kqueue_t::reset_pollin (handle_t handle_)
 {
-    poll_entry_t *pe = (poll_entry_t*) handle_;
+    check_thread ();
+    poll_entry_t *pe = (poll_entry_t *) handle_;
     if (likely (pe->flag_pollin)) {
         pe->flag_pollin = false;
         kevent_delete (pe->fd, EVFILT_READ);
@@ -126,7 +143,8 @@ void zmq::kqueue_t::reset_pollin (handle_t handle_)
 
 void zmq::kqueue_t::set_pollout (handle_t handle_)
 {
-    poll_entry_t *pe = (poll_entry_t*) handle_;
+    check_thread ();
+    poll_entry_t *pe = (poll_entry_t *) handle_;
     if (likely (!pe->flag_pollout)) {
         pe->flag_pollout = true;
         kevent_add (pe->fd, EVFILT_WRITE, pe);
@@ -135,21 +153,16 @@ void zmq::kqueue_t::set_pollout (handle_t handle_)
 
 void zmq::kqueue_t::reset_pollout (handle_t handle_)
 {
-    poll_entry_t *pe = (poll_entry_t*) handle_;
+    check_thread ();
+    poll_entry_t *pe = (poll_entry_t *) handle_;
     if (likely (pe->flag_pollout)) {
         pe->flag_pollout = false;
         kevent_delete (pe->fd, EVFILT_WRITE);
-   }
-}
-
-void zmq::kqueue_t::start ()
-{
-    worker.start (worker_routine, this);
+    }
 }
 
 void zmq::kqueue_t::stop ()
 {
-    stopping = true;
 }
 
 int zmq::kqueue_t::max_fds ()
@@ -159,18 +172,25 @@ int zmq::kqueue_t::max_fds ()
 
 void zmq::kqueue_t::loop ()
 {
-    while (!stopping) {
-
+    while (true) {
         //  Execute any due timers.
         int timeout = (int) execute_timers ();
 
+        if (get_load () == 0) {
+            if (timeout == 0)
+                break;
+
+            // TODO sleep for timeout
+            continue;
+        }
+
         //  Wait for events.
-        struct kevent ev_buf [max_io_events];
+        struct kevent ev_buf[max_io_events];
         timespec ts = {timeout / 1000, (timeout % 1000) * 1000000};
-        int n = kevent (kqueue_fd, NULL, 0, &ev_buf [0], max_io_events,
-            timeout ? &ts: NULL);
+        int n = kevent (kqueue_fd, NULL, 0, &ev_buf[0], max_io_events,
+                        timeout ? &ts : NULL);
 #ifdef HAVE_FORK
-        if (unlikely(pid != getpid())) {
+        if (unlikely (pid != getpid ())) {
             //printf("zmq::kqueue_t::loop aborting on forked child %d\n", (int)getpid());
             // simply exit the loop in a forked process.
             return;
@@ -181,34 +201,30 @@ void zmq::kqueue_t::loop ()
             continue;
         }
 
-        for (int i = 0; i < n; i ++) {
-            poll_entry_t *pe = (poll_entry_t*) ev_buf [i].udata;
+        for (int i = 0; i < n; i++) {
+            poll_entry_t *pe = (poll_entry_t *) ev_buf[i].udata;
 
             if (pe->fd == retired_fd)
                 continue;
-            if (ev_buf [i].flags & EV_EOF)
+            if (ev_buf[i].flags & EV_EOF)
                 pe->reactor->in_event ();
             if (pe->fd == retired_fd)
                 continue;
-            if (ev_buf [i].filter == EVFILT_WRITE)
+            if (ev_buf[i].filter == EVFILT_WRITE)
                 pe->reactor->out_event ();
             if (pe->fd == retired_fd)
                 continue;
-            if (ev_buf [i].filter == EVFILT_READ)
+            if (ev_buf[i].filter == EVFILT_READ)
                 pe->reactor->in_event ();
         }
 
         //  Destroy retired event sources.
         for (retired_t::iterator it = retired.begin (); it != retired.end ();
-              ++it)
-            delete *it;
+             ++it) {
+            LIBZMQ_DELETE (*it);
+        }
         retired.clear ();
     }
 }
 
-void zmq::kqueue_t::worker_routine (void *arg_)
-{
-    ((kqueue_t*) arg_)->loop ();
-}
-
 #endif
