@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2016 Contributors as noted in the AUTHORS file
 
     This file is part of libzmq, the ZeroMQ core engine in C++.
 
@@ -27,25 +27,179 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "precompiled.hpp"
 #include <string.h>
 
 #include "options.hpp"
 #include "err.hpp"
-#include "../include/zmq_utils.h"
+#include "macros.hpp"
+
+#ifndef ZMQ_HAVE_WINDOWS
+#include <net/if.h>
+#endif
+
+#if defined IFNAMSIZ
+#define BINDDEVSIZ IFNAMSIZ
+#else
+#define BINDDEVSIZ 16
+#endif
+
+static int sockopt_invalid ()
+{
+#if defined(ZMQ_ACT_MILITANT)
+    zmq_assert (false);
+#endif
+    errno = EINVAL;
+    return -1;
+}
+
+int zmq::do_getsockopt (void *const optval_,
+                        size_t *const optvallen_,
+                        const std::string &value_)
+{
+    return do_getsockopt (optval_, optvallen_, value_.c_str (),
+                          value_.size () + 1);
+}
+
+int zmq::do_getsockopt (void *const optval_,
+                        size_t *const optvallen_,
+                        const void *value_,
+                        const size_t value_len_)
+{
+    // TODO behaviour is inconsistent with options_t::getsockopt; there, an
+    // *exact* length match is required except for string-like (but not the
+    // CURVE keys!) (and therefore null-ing remaining memory is a no-op, see
+    // comment below)
+    if (*optvallen_ < value_len_) {
+        return sockopt_invalid ();
+    }
+    memcpy (optval_, value_, value_len_);
+    // TODO why is the remaining memory null-ed?
+    memset ((char *) optval_ + value_len_, 0, *optvallen_ - value_len_);
+    *optvallen_ = value_len_;
+    return 0;
+}
+
+#ifdef ZMQ_HAVE_CURVE
+static int do_getsockopt_curve_key (void *const optval_,
+                                    size_t *const optvallen_,
+                                    const uint8_t (&curve_key_)[CURVE_KEYSIZE])
+{
+    if (*optvallen_ == CURVE_KEYSIZE) {
+        memcpy (optval_, curve_key_, CURVE_KEYSIZE);
+        return 0;
+    } else if (*optvallen_ == CURVE_KEYSIZE_Z85 + 1) {
+        zmq_z85_encode ((char *) optval_, curve_key_, CURVE_KEYSIZE);
+        return 0;
+    }
+    return sockopt_invalid ();
+}
+#endif
+
+template <typename T>
+int do_setsockopt (const void *const optval_,
+                   const size_t optvallen_,
+                   T *const out_value_)
+{
+    if (optvallen_ == sizeof (T)) {
+        memcpy (out_value_, optval_, sizeof (T));
+        return 0;
+    }
+    return sockopt_invalid ();
+}
+
+int zmq::do_setsockopt_int_as_bool_strict (const void *const optval_,
+                                           const size_t optvallen_,
+                                           bool *const out_value_)
+{
+    // TODO handling of values other than 0 or 1 is not consistent,
+    // here it is disallowed, but for other options such as
+    // ZMQ_ROUTER_RAW any positive value is accepted
+    int value = -1;
+    if (do_setsockopt (optval_, optvallen_, &value) == -1)
+        return -1;
+    if (value == 0 || value == 1) {
+        *out_value_ = (value != 0);
+        return 0;
+    }
+    return sockopt_invalid ();
+}
+
+int zmq::do_setsockopt_int_as_bool_relaxed (const void *const optval_,
+                                            const size_t optvallen_,
+                                            bool *const out_value_)
+{
+    int value = -1;
+    if (do_setsockopt (optval_, optvallen_, &value) == -1)
+        return -1;
+    *out_value_ = (value != 0);
+    return 0;
+}
+
+static int
+do_setsockopt_string_allow_empty_strict (const void *const optval_,
+                                         const size_t optvallen_,
+                                         std::string *const out_value_,
+                                         const size_t max_len_)
+{
+    // TODO why is optval_ != NULL not allowed in case of optvallen_== 0?
+    // TODO why are empty strings allowed for some socket options, but not for others?
+    if (optval_ == NULL && optvallen_ == 0) {
+        out_value_->clear ();
+        return 0;
+    } else if (optval_ != NULL && optvallen_ > 0 && optvallen_ <= max_len_) {
+        out_value_->assign ((const char *) optval_, optvallen_);
+        return 0;
+    }
+    return sockopt_invalid ();
+}
+
+static int
+do_setsockopt_string_allow_empty_relaxed (const void *const optval_,
+                                          const size_t optvallen_,
+                                          std::string *const out_value_,
+                                          const size_t max_len_)
+{
+    // TODO use either do_setsockopt_string_allow_empty_relaxed or
+    // do_setsockopt_string_allow_empty_strict everywhere
+    if (optvallen_ > 0 && optvallen_ <= max_len_) {
+        out_value_->assign ((const char *) optval_, optvallen_);
+        return 0;
+    }
+    return sockopt_invalid ();
+}
+
+template <typename T>
+int do_setsockopt_set (const void *const optval_,
+                       const size_t optvallen_,
+                       std::set<T> *const set_)
+{
+    if (optvallen_ == 0 && optval_ == NULL) {
+        set_->clear ();
+        return 0;
+    } else if (optvallen_ == sizeof (T) && optval_ != NULL) {
+        set_->insert (*((const T *) optval_));
+        return 0;
+    }
+    return sockopt_invalid ();
+}
 
 zmq::options_t::options_t () :
     sndhwm (1000),
     rcvhwm (1000),
     affinity (0),
-    identity_size (0),
+    routing_id_size (0),
     rate (100),
     recovery_ivl (10000),
     multicast_hops (1),
-    sndbuf (0),
-    rcvbuf (0),
+    multicast_maxtpdu (1500),
+    sndbuf (-1),
+    rcvbuf (-1),
     tos (0),
     type (-1),
     linger (-1),
+    connect_timeout (0),
+    tcp_maxrt (0),
     reconnect_ivl (100),
     reconnect_ivl_max (0),
     backlog (100),
@@ -55,28 +209,85 @@ zmq::options_t::options_t () :
     ipv6 (0),
     immediate (0),
     filter (false),
-    recv_identity (false),
-    raw_sock (false),
+    invert_matching (false),
+    recv_routing_id (false),
+    raw_socket (false),
+    raw_notify (true),
     tcp_keepalive (-1),
     tcp_keepalive_cnt (-1),
     tcp_keepalive_idle (-1),
     tcp_keepalive_intvl (-1),
     mechanism (ZMQ_NULL),
     as_server (0),
+    gss_principal_nt (ZMQ_GSSAPI_NT_HOSTBASED),
+    gss_service_principal_nt (ZMQ_GSSAPI_NT_HOSTBASED),
     gss_plaintext (false),
     socket_id (0),
     conflate (false),
-    handshake_ivl (30000)
+    handshake_ivl (30000),
+    connected (false),
+    heartbeat_ttl (0),
+    heartbeat_interval (0),
+    heartbeat_timeout (-1),
+    use_fd (-1),
+    zap_enforce_domain (false),
+    loopback_fastpath (false),
+    zero_copy (true)
 {
+    memset (curve_public_key, 0, CURVE_KEYSIZE);
+    memset (curve_secret_key, 0, CURVE_KEYSIZE);
+    memset (curve_server_key, 0, CURVE_KEYSIZE);
+#if defined ZMQ_HAVE_VMCI
+    vmci_buffer_size = 0;
+    vmci_buffer_min_size = 0;
+    vmci_buffer_max_size = 0;
+    vmci_connect_timeout = -1;
+#endif
 }
 
-int zmq::options_t::setsockopt (int option_, const void *optval_,
-    size_t optvallen_)
+int zmq::options_t::set_curve_key (uint8_t *destination,
+                                   const void *optval_,
+                                   size_t optvallen_)
+{
+    switch (optvallen_) {
+        case CURVE_KEYSIZE:
+            memcpy (destination, optval_, optvallen_);
+            mechanism = ZMQ_CURVE;
+            return 0;
+
+        case CURVE_KEYSIZE_Z85 + 1:
+            if (zmq_z85_decode (destination, (char *) optval_)) {
+                mechanism = ZMQ_CURVE;
+                return 0;
+            }
+            break;
+
+        case CURVE_KEYSIZE_Z85:
+            char z85_key[CURVE_KEYSIZE_Z85 + 1];
+            memcpy (z85_key, (char *) optval_, optvallen_);
+            z85_key[CURVE_KEYSIZE_Z85] = 0;
+            if (zmq_z85_decode (destination, z85_key)) {
+                mechanism = ZMQ_CURVE;
+                return 0;
+            }
+            break;
+
+        default:
+            break;
+    }
+    return -1;
+}
+
+int zmq::options_t::setsockopt (int option_,
+                                const void *optval_,
+                                size_t optvallen_)
 {
     bool is_int = (optvallen_ == sizeof (int));
-    int value = is_int? *((int *) optval_): 0;
-#if defined (ZMQ_ACT_MILITANT)
-    bool malformed = true;          //  Did caller pass a bad option value?
+    int value = 0;
+    if (is_int)
+        memcpy (&value, optval_, sizeof (int));
+#if defined(ZMQ_ACT_MILITANT)
+    bool malformed = true; //  Did caller pass a bad option value?
 #endif
 
     switch (option_) {
@@ -95,17 +306,13 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
             break;
 
         case ZMQ_AFFINITY:
-            if (optvallen_ == sizeof (uint64_t)) {
-                affinity = *((uint64_t*) optval_);
-                return 0;
-            }
-            break;
+            return do_setsockopt (optval_, optvallen_, &affinity);
 
-        case ZMQ_IDENTITY:
-            //  Identity is any binary string from 1 to 255 octets
+        case ZMQ_ROUTING_ID:
+            //  Routing id is any binary string from 1 to 255 octets
             if (optvallen_ > 0 && optvallen_ < 256) {
-                identity_size = optvallen_;
-                memcpy (identity, optval_, identity_size);
+                routing_id_size = (unsigned char) optvallen_;
+                memcpy (routing_id, optval_, routing_id_size);
                 return 0;
             }
             break;
@@ -125,14 +332,14 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
             break;
 
         case ZMQ_SNDBUF:
-            if (is_int && value >= 0) {
+            if (is_int && value >= -1) {
                 sndbuf = value;
                 return 0;
             }
             break;
 
         case ZMQ_RCVBUF:
-            if (is_int && value >= 0) {
+            if (is_int && value >= -1) {
                 rcvbuf = value;
                 return 0;
             }
@@ -147,7 +354,21 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
 
         case ZMQ_LINGER:
             if (is_int && value >= -1) {
-                linger = value;
+                linger.store (value);
+                return 0;
+            }
+            break;
+
+        case ZMQ_CONNECT_TIMEOUT:
+            if (is_int && value >= 0) {
+                connect_timeout = value;
+                return 0;
+            }
+            break;
+
+        case ZMQ_TCP_MAXRT:
+            if (is_int && value >= 0) {
+                tcp_maxrt = value;
                 return 0;
             }
             break;
@@ -174,11 +395,7 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
             break;
 
         case ZMQ_MAXMSGSIZE:
-            if (optvallen_ == sizeof (int64_t)) {
-                maxmsgsize = *((int64_t *) optval_);
-                return 0;
-            }
-            break;
+            return do_setsockopt (optval_, optvallen_, &maxmsgsize);
 
         case ZMQ_MULTICAST_HOPS:
             if (is_int && value > 0) {
@@ -187,6 +404,13 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
             }
             break;
 
+        case ZMQ_MULTICAST_MAXTPDU:
+            if (is_int && value > 0) {
+                multicast_maxtpdu = value;
+                return 0;
+            }
+            break;
+
         case ZMQ_RCVTIMEO:
             if (is_int && value >= -1) {
                 rcvtimeo = value;
@@ -202,33 +426,23 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
             break;
 
         /*  Deprecated in favor of ZMQ_IPV6  */
-        case ZMQ_IPV4ONLY:
-            if (is_int && (value == 0 || value == 1)) {
-                ipv6 = (value == 0);
-                return 0;
-            }
-            break;
+        case ZMQ_IPV4ONLY: {
+            bool value;
+            int rc =
+              do_setsockopt_int_as_bool_strict (optval_, optvallen_, &value);
+            if (rc == 0)
+                ipv6 = !value;
+            return rc;
+        }
 
         /*  To replace the somewhat surprising IPV4ONLY */
         case ZMQ_IPV6:
-            if (is_int && (value == 0 || value == 1)) {
-                ipv6 = (value != 0);
-                return 0;
-            }
-            break;
+            return do_setsockopt_int_as_bool_strict (optval_, optvallen_,
+                                                     &ipv6);
 
         case ZMQ_SOCKS_PROXY:
-            if (optval_ == NULL && optvallen_ == 0) {
-                socks_proxy_address.clear ();
-                return 0;
-            }
-            else
-            if (optval_ != NULL && optvallen_ > 0 ) {
-                socks_proxy_address =
-                    std::string ((const char *) optval_, optvallen_);
-                return 0;
-            }
-            break;
+            return do_setsockopt_string_allow_empty_strict (
+              optval_, optvallen_, &socks_proxy_address, SIZE_MAX);
 
         case ZMQ_TCP_KEEPALIVE:
             if (is_int && (value == -1 || value == 0 || value == 1)) {
@@ -259,73 +473,52 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
             break;
 
         case ZMQ_IMMEDIATE:
+            // TODO why is immediate not bool (and called non_immediate, as its meaning appears to be reversed)
             if (is_int && (value == 0 || value == 1)) {
                 immediate = value;
                 return 0;
             }
             break;
 
-        case ZMQ_TCP_ACCEPT_FILTER:
-            if (optvallen_ == 0 && optval_ == NULL) {
-                tcp_accept_filters.clear ();
-                return 0;
-            }
-            else
-            if (optvallen_ > 0 && optvallen_ < 256 && optval_ != NULL && *((const char*) optval_) != 0) {
-                std::string filter_str ((const char *) optval_, optvallen_);
-                tcp_address_mask_t mask;
-                int rc = mask.resolve (filter_str.c_str (), ipv6);
-                if (rc == 0) {
-                    tcp_accept_filters.push_back (mask);
-                    return 0;
+        case ZMQ_TCP_ACCEPT_FILTER: {
+            std::string filter_str;
+            int rc = do_setsockopt_string_allow_empty_strict (
+              optval_, optvallen_, &filter_str, 255);
+            if (rc == 0) {
+                if (filter_str.empty ()) {
+                    tcp_accept_filters.clear ();
+                } else {
+                    tcp_address_mask_t mask;
+                    rc = mask.resolve (filter_str.c_str (), ipv6);
+                    if (rc == 0) {
+                        tcp_accept_filters.push_back (mask);
+                    }
                 }
             }
-            break;
+            return rc;
+        }
 
-#       if defined ZMQ_HAVE_SO_PEERCRED || defined ZMQ_HAVE_LOCAL_PEERCRED
+#if defined ZMQ_HAVE_SO_PEERCRED || defined ZMQ_HAVE_LOCAL_PEERCRED
         case ZMQ_IPC_FILTER_UID:
-            if (optvallen_ == 0 && optval_ == NULL) {
-                ipc_uid_accept_filters.clear ();
-                return 0;
-            }
-            else
-            if (optvallen_ == sizeof (uid_t) && optval_ != NULL) {
-                ipc_uid_accept_filters.insert (*((uid_t *) optval_));
-                return 0;
-            }
-            break;
+            return do_setsockopt_set (optval_, optvallen_,
+                                      &ipc_uid_accept_filters);
+
 
         case ZMQ_IPC_FILTER_GID:
-            if (optvallen_ == 0 && optval_ == NULL) {
-                ipc_gid_accept_filters.clear ();
-                return 0;
-            }
-            else
-            if (optvallen_ == sizeof (gid_t) && optval_ != NULL) {
-                ipc_gid_accept_filters.insert (*((gid_t *) optval_));
-                return 0;
-            }
-            break;
-#       endif
+            return do_setsockopt_set (optval_, optvallen_,
+                                      &ipc_gid_accept_filters);
+#endif
 
-#       if defined ZMQ_HAVE_SO_PEERCRED
+#if defined ZMQ_HAVE_SO_PEERCRED
         case ZMQ_IPC_FILTER_PID:
-            if (optvallen_ == 0 && optval_ == NULL) {
-                ipc_pid_accept_filters.clear ();
-                return 0;
-            }
-            else
-            if (optvallen_ == sizeof (pid_t) && optval_ != NULL) {
-                ipc_pid_accept_filters.insert (*((pid_t *) optval_));
-                return 0;
-            }
-            break;
-#       endif
+            return do_setsockopt_set (optval_, optvallen_,
+                                      &ipc_pid_accept_filters);
+#endif
 
         case ZMQ_PLAIN_SERVER:
             if (is_int && (value == 0 || value == 1)) {
                 as_server = value;
-                mechanism = value? ZMQ_PLAIN: ZMQ_NULL;
+                mechanism = value ? ZMQ_PLAIN : ZMQ_NULL;
                 return 0;
             }
             break;
@@ -334,9 +527,7 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
             if (optvallen_ == 0 && optval_ == NULL) {
                 mechanism = ZMQ_NULL;
                 return 0;
-            }
-            else
-            if (optvallen_ > 0 && optvallen_ < 256 && optval_ != NULL) {
+            } else if (optvallen_ > 0 && optvallen_ < 256 && optval_ != NULL) {
                 plain_username.assign ((const char *) optval_, optvallen_);
                 as_server = 0;
                 mechanism = ZMQ_PLAIN;
@@ -348,9 +539,7 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
             if (optvallen_ == 0 && optval_ == NULL) {
                 mechanism = ZMQ_NULL;
                 return 0;
-            }
-            else
-            if (optvallen_ > 0 && optvallen_ < 256 && optval_ != NULL) {
+            } else if (optvallen_ > 0 && optvallen_ < 256 && optval_ != NULL) {
                 plain_password.assign ((const char *) optval_, optvallen_);
                 as_server = 0;
                 mechanism = ZMQ_PLAIN;
@@ -359,110 +548,46 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
             break;
 
         case ZMQ_ZAP_DOMAIN:
-            if (optvallen_ < 256) {
-                zap_domain.assign ((const char *) optval_, optvallen_);
-                return 0;
-            }
+            return do_setsockopt_string_allow_empty_relaxed (
+              optval_, optvallen_, &zap_domain, 255);
             break;
 
-        //  If libsodium isn't installed, these options provoke EINVAL
-#       ifdef HAVE_LIBSODIUM
+            //  If curve encryption isn't built, these options provoke EINVAL
+#ifdef ZMQ_HAVE_CURVE
         case ZMQ_CURVE_SERVER:
             if (is_int && (value == 0 || value == 1)) {
                 as_server = value;
-                mechanism = value? ZMQ_CURVE: ZMQ_NULL;
+                mechanism = value ? ZMQ_CURVE : ZMQ_NULL;
                 return 0;
             }
             break;
 
         case ZMQ_CURVE_PUBLICKEY:
-            //  TODO: refactor repeated code for these three options
-            //  into set_curve_key (destination, optval, optlen) method
-            //  ==> set_curve_key (curve_public_key, optval_, optvallen_);
-            if (optvallen_ == CURVE_KEYSIZE) {
-                memcpy (curve_public_key, optval_, CURVE_KEYSIZE);
-                mechanism = ZMQ_CURVE;
-                return 0;
-            }
-            else
-            if (optvallen_ == CURVE_KEYSIZE_Z85 + 1) {
-                zmq_z85_decode (curve_public_key, (char *) optval_);
-                mechanism = ZMQ_CURVE;
-                return 0;
-            }
-            else
-            //  Deprecated, not symmetrical with zmq_getsockopt
-            if (optvallen_ == CURVE_KEYSIZE_Z85) {
-                char z85_key [41];
-                memcpy (z85_key, (char *) optval_, CURVE_KEYSIZE_Z85);
-                z85_key [CURVE_KEYSIZE_Z85] = 0;
-                zmq_z85_decode (curve_public_key, z85_key);
-                mechanism = ZMQ_CURVE;
+            if (0 == set_curve_key (curve_public_key, optval_, optvallen_)) {
                 return 0;
             }
             break;
 
         case ZMQ_CURVE_SECRETKEY:
-            if (optvallen_ == CURVE_KEYSIZE) {
-                memcpy (curve_secret_key, optval_, CURVE_KEYSIZE);
-                mechanism = ZMQ_CURVE;
-                return 0;
-            }
-            else
-            if (optvallen_ == CURVE_KEYSIZE_Z85 + 1) {
-                zmq_z85_decode (curve_secret_key, (char *) optval_);
-                mechanism = ZMQ_CURVE;
-                return 0;
-            }
-            else
-            //  Deprecated, not symmetrical with zmq_getsockopt
-            if (optvallen_ == CURVE_KEYSIZE_Z85) {
-                char z85_key [41];
-                memcpy (z85_key, (char *) optval_, CURVE_KEYSIZE_Z85);
-                z85_key [CURVE_KEYSIZE_Z85] = 0;
-                zmq_z85_decode (curve_secret_key, z85_key);
-                mechanism = ZMQ_CURVE;
+            if (0 == set_curve_key (curve_secret_key, optval_, optvallen_)) {
                 return 0;
             }
             break;
 
         case ZMQ_CURVE_SERVERKEY:
-            if (optvallen_ == CURVE_KEYSIZE) {
-                memcpy (curve_server_key, optval_, CURVE_KEYSIZE);
-                mechanism = ZMQ_CURVE;
-                as_server = 0;
-                return 0;
-            }
-            else
-            if (optvallen_ == CURVE_KEYSIZE_Z85 + 1) {
-                zmq_z85_decode (curve_server_key, (char *) optval_);
-                mechanism = ZMQ_CURVE;
-                as_server = 0;
-                return 0;
-            }
-            else
-            //  Deprecated, not symmetrical with zmq_getsockopt
-            if (optvallen_ == CURVE_KEYSIZE_Z85) {
-                char z85_key [41];
-                memcpy (z85_key, (char *) optval_, CURVE_KEYSIZE_Z85);
-                z85_key [CURVE_KEYSIZE_Z85] = 0;
-                zmq_z85_decode (curve_server_key, z85_key);
-                mechanism = ZMQ_CURVE;
+            if (0 == set_curve_key (curve_server_key, optval_, optvallen_)) {
                 as_server = 0;
                 return 0;
             }
             break;
-#       endif
+#endif
 
         case ZMQ_CONFLATE:
-            if (is_int && (value == 0 || value == 1)) {
-                conflate = (value != 0);
-                return 0;
-            }
-            break;
+            return do_setsockopt_int_as_bool_strict (optval_, optvallen_,
+                                                     &conflate);
 
-        //  If libgssapi isn't installed, these options provoke EINVAL
-#       ifdef HAVE_LIBGSSAPI_KRB5
+            //  If libgssapi isn't installed, these options provoke EINVAL
+#ifdef HAVE_LIBGSSAPI_KRB5
         case ZMQ_GSSAPI_SERVER:
             if (is_int && (value == 0 || value == 1)) {
                 as_server = value;
@@ -481,7 +606,8 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
 
         case ZMQ_GSSAPI_SERVICE_PRINCIPAL:
             if (optvallen_ > 0 && optvallen_ < 256 && optval_ != NULL) {
-                gss_service_principal.assign ((const char *) optval_, optvallen_);
+                gss_service_principal.assign ((const char *) optval_,
+                                              optvallen_);
                 mechanism = ZMQ_GSSAPI;
                 as_server = 0;
                 return 0;
@@ -489,12 +615,29 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
             break;
 
         case ZMQ_GSSAPI_PLAINTEXT:
-            if (is_int && (value == 0 || value == 1)) {
-                gss_plaintext = (value != 0);
+            return do_setsockopt_int_as_bool_strict (optval_, optvallen_,
+                                                     &gss_plaintext);
+
+        case ZMQ_GSSAPI_PRINCIPAL_NAMETYPE:
+            if (is_int
+                && (value == ZMQ_GSSAPI_NT_HOSTBASED
+                    || value == ZMQ_GSSAPI_NT_USER_NAME
+                    || value == ZMQ_GSSAPI_NT_KRB5_PRINCIPAL)) {
+                gss_principal_nt = value;
                 return 0;
             }
             break;
-#       endif
+
+        case ZMQ_GSSAPI_SERVICE_PRINCIPAL_NAMETYPE:
+            if (is_int
+                && (value == ZMQ_GSSAPI_NT_HOSTBASED
+                    || value == ZMQ_GSSAPI_NT_USER_NAME
+                    || value == ZMQ_GSSAPI_NT_KRB5_PRINCIPAL)) {
+                gss_service_principal_nt = value;
+                return 0;
+            }
+            break;
+#endif
 
         case ZMQ_HANDSHAKE_IVL:
             if (is_int && value >= 0) {
@@ -503,8 +646,88 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
             }
             break;
 
+        case ZMQ_INVERT_MATCHING:
+            return do_setsockopt_int_as_bool_relaxed (optval_, optvallen_,
+                                                      &invert_matching);
+
+        case ZMQ_HEARTBEAT_IVL:
+            if (is_int && value >= 0) {
+                heartbeat_interval = value;
+                return 0;
+            }
+            break;
+
+        case ZMQ_HEARTBEAT_TTL:
+            // Convert this to deciseconds from milliseconds
+            value = value / 100;
+            if (is_int && value >= 0 && value <= 6553) {
+                heartbeat_ttl = (uint16_t) value;
+                return 0;
+            }
+            break;
+
+        case ZMQ_HEARTBEAT_TIMEOUT:
+            if (is_int && value >= 0) {
+                heartbeat_timeout = value;
+                return 0;
+            }
+            break;
+
+#ifdef ZMQ_HAVE_VMCI
+        case ZMQ_VMCI_BUFFER_SIZE:
+            return do_setsockopt (optval_, optvallen_, &vmci_buffer_size);
+
+        case ZMQ_VMCI_BUFFER_MIN_SIZE:
+            return do_setsockopt (optval_, optvallen_, &vmci_buffer_min_size);
+
+        case ZMQ_VMCI_BUFFER_MAX_SIZE:
+            return do_setsockopt (optval_, optvallen_, &vmci_buffer_max_size);
+
+        case ZMQ_VMCI_CONNECT_TIMEOUT:
+            return do_setsockopt (optval_, optvallen_, &vmci_connect_timeout);
+#endif
+
+        case ZMQ_USE_FD:
+            if (is_int && value >= -1) {
+                use_fd = value;
+                return 0;
+            }
+            break;
+
+        case ZMQ_BINDTODEVICE:
+            return do_setsockopt_string_allow_empty_strict (
+              optval_, optvallen_, &bound_device, BINDDEVSIZ);
+
+        case ZMQ_ZAP_ENFORCE_DOMAIN:
+            return do_setsockopt_int_as_bool_relaxed (optval_, optvallen_,
+                                                      &zap_enforce_domain);
+
+        case ZMQ_LOOPBACK_FASTPATH:
+            return do_setsockopt_int_as_bool_relaxed (optval_, optvallen_,
+                                                      &loopback_fastpath);
+
+        case ZMQ_METADATA:
+            if (optvallen_ > 0 && !is_int) {
+                std::string s ((char *) optval_);
+                size_t pos = 0;
+                std::string key, val, delimiter = ":";
+                pos = s.find (delimiter);
+                if (pos != std::string::npos && pos != 0
+                    && pos != s.length () - 1) {
+                    key = s.substr (0, pos);
+                    if (key.compare (0, 2, "X-") == 0 && key.length () < 256) {
+                        val = s.substr (pos + 1, s.length ());
+                        app_metadata.insert (
+                          std::pair<std::string, std::string> (key, val));
+                        return 0;
+                    }
+                }
+            }
+            errno = EINVAL;
+            return -1;
+            break;
         default:
-#if defined (ZMQ_ACT_MILITANT)
+#if defined(ZMQ_ACT_MILITANT)
             //  There are valid scenarios for probing with unknown socket option
             //  values, e.g. to check if security is enabled or not. This will not
             //  provoke a militant assert. However, passing bad values to a valid
@@ -513,7 +736,13 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
 #endif
             break;
     }
-#if defined (ZMQ_ACT_MILITANT)
+
+            // TODO mechanism should either be set explicitly, or determined when
+            // connecting. currently, it depends on the order of setsockopt calls
+            // if there is some inconsistency, which is confusing. in addition,
+            // the assumed or set mechanism should be queryable (as a socket option)
+
+#if defined(ZMQ_ACT_MILITANT)
     //  There is no valid use case for passing an error back to the application
     //  when it sent malformed arguments to a socket option. Use ./configure
     //  --with-militant to enable this checking.
@@ -524,12 +753,14 @@ int zmq::options_t::setsockopt (int option_, const void *optval_,
     return -1;
 }
 
-int zmq::options_t::getsockopt (int option_, void *optval_, size_t *optvallen_)
+int zmq::options_t::getsockopt (int option_,
+                                void *optval_,
+                                size_t *optvallen_) const
 {
     bool is_int = (*optvallen_ == sizeof (int));
     int *value = (int *) optval_;
-#if defined (ZMQ_ACT_MILITANT)
-    bool malformed = true;          //  Did caller pass a bad option value?
+#if defined(ZMQ_ACT_MILITANT)
+    bool malformed = true; //  Did caller pass a bad option value?
 #endif
 
     switch (option_) {
@@ -554,12 +785,9 @@ int zmq::options_t::getsockopt (int option_, void *optval_, size_t *optvallen_)
             }
             break;
 
-        case ZMQ_IDENTITY:
-            if (*optvallen_ >= identity_size) {
-                memcpy (optval_, identity, identity_size);
-                *optvallen_ = identity_size;
-                return 0;
-            }
+        case ZMQ_ROUTING_ID:
+            return do_getsockopt (optval_, optvallen_, routing_id,
+                                  routing_id_size);
             break;
 
         case ZMQ_RATE:
@@ -606,7 +834,21 @@ int zmq::options_t::getsockopt (int option_, void *optval_, size_t *optvallen_)
 
         case ZMQ_LINGER:
             if (is_int) {
-                *value = linger;
+                *value = linger.load ();
+                return 0;
+            }
+            break;
+
+        case ZMQ_CONNECT_TIMEOUT:
+            if (is_int) {
+                *value = connect_timeout;
+                return 0;
+            }
+            break;
+
+        case ZMQ_TCP_MAXRT:
+            if (is_int) {
+                *value = tcp_maxrt;
                 return 0;
             }
             break;
@@ -647,6 +889,13 @@ int zmq::options_t::getsockopt (int option_, void *optval_, size_t *optvallen_)
             }
             break;
 
+        case ZMQ_MULTICAST_MAXTPDU:
+            if (is_int) {
+                *value = multicast_maxtpdu;
+                return 0;
+            }
+            break;
+
         case ZMQ_RCVTIMEO:
             if (is_int) {
                 *value = rcvtimeo;
@@ -683,11 +932,7 @@ int zmq::options_t::getsockopt (int option_, void *optval_, size_t *optvallen_)
             break;
 
         case ZMQ_SOCKS_PROXY:
-            if (*optvallen_ >= socks_proxy_address.size () + 1) {
-                memcpy (optval_, socks_proxy_address.c_str (), socks_proxy_address.size () + 1);
-                *optvallen_ = socks_proxy_address.size () + 1;
-                return 0;
-            }
+            return do_getsockopt (optval_, optvallen_, socks_proxy_address);
             break;
 
         case ZMQ_TCP_KEEPALIVE:
@@ -733,31 +978,19 @@ int zmq::options_t::getsockopt (int option_, void *optval_, size_t *optvallen_)
             break;
 
         case ZMQ_PLAIN_USERNAME:
-            if (*optvallen_ >= plain_username.size () + 1) {
-                memcpy (optval_, plain_username.c_str (), plain_username.size () + 1);
-                *optvallen_ = plain_username.size () + 1;
-                return 0;
-            }
+            return do_getsockopt (optval_, optvallen_, plain_username);
             break;
 
         case ZMQ_PLAIN_PASSWORD:
-            if (*optvallen_ >= plain_password.size () + 1) {
-                memcpy (optval_, plain_password.c_str (), plain_password.size () + 1);
-                *optvallen_ = plain_password.size () + 1;
-                return 0;
-            }
+            return do_getsockopt (optval_, optvallen_, plain_password);
             break;
 
         case ZMQ_ZAP_DOMAIN:
-            if (*optvallen_ >= zap_domain.size () + 1) {
-                memcpy (optval_, zap_domain.c_str (), zap_domain.size () + 1);
-                *optvallen_ = zap_domain.size () + 1;
-                return 0;
-            }
+            return do_getsockopt (optval_, optvallen_, zap_domain);
             break;
 
-        //  If libsodium isn't installed, these options provoke EINVAL
-#       ifdef HAVE_LIBSODIUM
+            //  If curve encryption isn't built, these options provoke EINVAL
+#ifdef ZMQ_HAVE_CURVE
         case ZMQ_CURVE_SERVER:
             if (is_int) {
                 *value = as_server && mechanism == ZMQ_CURVE;
@@ -766,41 +999,20 @@ int zmq::options_t::getsockopt (int option_, void *optval_, size_t *optvallen_)
             break;
 
         case ZMQ_CURVE_PUBLICKEY:
-            if (*optvallen_ == CURVE_KEYSIZE) {
-                memcpy (optval_, curve_public_key, CURVE_KEYSIZE);
-                return 0;
-            }
-            else
-            if (*optvallen_ == CURVE_KEYSIZE_Z85 + 1) {
-                zmq_z85_encode ((char *) optval_, curve_public_key, CURVE_KEYSIZE);
-                return 0;
-            }
+            return do_getsockopt_curve_key (optval_, optvallen_,
+                                            curve_public_key);
             break;
 
         case ZMQ_CURVE_SECRETKEY:
-            if (*optvallen_ == CURVE_KEYSIZE) {
-                memcpy (optval_, curve_secret_key, CURVE_KEYSIZE);
-                return 0;
-            }
-            else
-            if (*optvallen_ == CURVE_KEYSIZE_Z85 + 1) {
-                zmq_z85_encode ((char *) optval_, curve_secret_key, CURVE_KEYSIZE);
-                return 0;
-            }
+            return do_getsockopt_curve_key (optval_, optvallen_,
+                                            curve_secret_key);
             break;
 
         case ZMQ_CURVE_SERVERKEY:
-            if (*optvallen_ == CURVE_KEYSIZE) {
-                memcpy (optval_, curve_server_key, CURVE_KEYSIZE);
-                return 0;
-            }
-            else
-            if (*optvallen_ == CURVE_KEYSIZE_Z85 + 1) {
-                zmq_z85_encode ((char *) optval_, curve_server_key, CURVE_KEYSIZE);
-                return 0;
-            }
+            return do_getsockopt_curve_key (optval_, optvallen_,
+                                            curve_server_key);
             break;
-#       endif
+#endif
 
         case ZMQ_CONFLATE:
             if (is_int) {
@@ -809,8 +1021,8 @@ int zmq::options_t::getsockopt (int option_, void *optval_, size_t *optvallen_)
             }
             break;
 
-        //  If libgssapi isn't installed, these options provoke EINVAL
-#       ifdef HAVE_LIBGSSAPI_KRB5
+            //  If libgssapi isn't installed, these options provoke EINVAL
+#ifdef HAVE_LIBGSSAPI_KRB5
         case ZMQ_GSSAPI_SERVER:
             if (is_int) {
                 *value = as_server && mechanism == ZMQ_GSSAPI;
@@ -819,19 +1031,11 @@ int zmq::options_t::getsockopt (int option_, void *optval_, size_t *optvallen_)
             break;
 
         case ZMQ_GSSAPI_PRINCIPAL:
-            if (*optvallen_ >= gss_principal.size () + 1) {
-                memcpy (optval_, gss_principal.c_str (), gss_principal.size () + 1);
-                *optvallen_ = gss_principal.size () + 1;
-                return 0;
-            }
+            return do_getsockopt (optval_, optvallen_, gss_principal);
             break;
 
         case ZMQ_GSSAPI_SERVICE_PRINCIPAL:
-            if (*optvallen_ >= gss_service_principal.size () + 1) {
-                memcpy (optval_, gss_service_principal.c_str (), gss_service_principal.size () + 1);
-                *optvallen_ = gss_service_principal.size () + 1;
-                return 0;
-            }
+            return do_getsockopt (optval_, optvallen_, gss_service_principal);
             break;
 
         case ZMQ_GSSAPI_PLAINTEXT:
@@ -840,6 +1044,19 @@ int zmq::options_t::getsockopt (int option_, void *optval_, size_t *optvallen_)
                 return 0;
             }
             break;
+
+        case ZMQ_GSSAPI_PRINCIPAL_NAMETYPE:
+            if (is_int) {
+                *value = gss_principal_nt;
+                return 0;
+            }
+            break;
+        case ZMQ_GSSAPI_SERVICE_PRINCIPAL_NAMETYPE:
+            if (is_int) {
+                *value = gss_service_principal_nt;
+                return 0;
+            }
+            break;
 #endif
 
         case ZMQ_HANDSHAKE_IVL:
@@ -849,16 +1066,76 @@ int zmq::options_t::getsockopt (int option_, void *optval_, size_t *optvallen_)
             }
             break;
 
+        case ZMQ_INVERT_MATCHING:
+            if (is_int) {
+                *value = invert_matching;
+                return 0;
+            }
+            break;
+
+        case ZMQ_HEARTBEAT_IVL:
+            if (is_int) {
+                *value = heartbeat_interval;
+                return 0;
+            }
+            break;
+
+        case ZMQ_HEARTBEAT_TTL:
+            if (is_int) {
+                // Convert the internal deciseconds value to milliseconds
+                *value = heartbeat_ttl * 100;
+                return 0;
+            }
+            break;
+
+        case ZMQ_HEARTBEAT_TIMEOUT:
+            if (is_int) {
+                *value = heartbeat_timeout;
+                return 0;
+            }
+            break;
+
+        case ZMQ_USE_FD:
+            if (is_int) {
+                *value = use_fd;
+                return 0;
+            }
+            break;
+
+        case ZMQ_BINDTODEVICE:
+            return do_getsockopt (optval_, optvallen_, bound_device);
+            break;
+
+        case ZMQ_ZAP_ENFORCE_DOMAIN:
+            if (is_int) {
+                *value = zap_enforce_domain;
+                return 0;
+            }
+            break;
+
+        case ZMQ_LOOPBACK_FASTPATH:
+            if (is_int) {
+                *value = loopback_fastpath;
+                return 0;
+            }
+            break;
+
         default:
-#if defined (ZMQ_ACT_MILITANT)
+#if defined(ZMQ_ACT_MILITANT)
             malformed = false;
 #endif
             break;
     }
-#if defined (ZMQ_ACT_MILITANT)
+#if defined(ZMQ_ACT_MILITANT)
     if (malformed)
         zmq_assert (false);
 #endif
     errno = EINVAL;
     return -1;
 }
+
+bool zmq::options_t::is_valid (int option_) const
+{
+    LIBZMQ_UNUSED (option_);
+    return true;
+}
