@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2016 Contributors as noted in the AUTHORS file
 
     This file is part of libzmq, the ZeroMQ core engine in C++.
 
@@ -27,6 +27,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "precompiled.hpp"
 #include "rep.hpp"
 #include "err.hpp"
 #include "msg.hpp"
@@ -91,8 +92,7 @@ int zmq::rep_t::xrecv (msg_t *msg_)
 
                 if (bottom)
                     break;
-            }
-            else {
+            } else {
                 //  If the traceback stack is malformed, discard anything
                 //  already sent to pipe (we're at end of invalid message).
                 rc = router_t::rollback ();
@@ -105,7 +105,7 @@ int zmq::rep_t::xrecv (msg_t *msg_)
     //  Get next message part to return to the user.
     int rc = router_t::xrecv (msg_);
     if (rc != 0)
-       return rc;
+        return rc;
 
     //  If whole request is read, flip the FSM to reply-sending state.
     if (!(msg_->flags () & msg_t::more)) {
