@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2016 Contributors as noted in the AUTHORS file
 
     This file is part of libzmq, the ZeroMQ core engine in C++.
 
@@ -27,8 +27,10 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "precompiled.hpp"
 #include <string.h>
 
+#include "macros.hpp"
 #include "xsub.hpp"
 #include "err.hpp"
 
@@ -41,7 +43,7 @@ zmq::xsub_t::xsub_t (class ctx_t *parent_, uint32_t tid_, int sid_) :
 
     //  When socket is being closed down we don't want to wait till pending
     //  subscription commands are sent to the wire.
-    options.linger = 0;
+    options.linger.store (0);
 
     int rc = message.init ();
     errno_assert (rc == 0);
@@ -55,8 +57,7 @@ zmq::xsub_t::~xsub_t ()
 
 void zmq::xsub_t::xattach_pipe (pipe_t *pipe_, bool subscribe_to_all_)
 {
-    // subscribe_to_all_ is unused
-    (void) subscribe_to_all_;
+    LIBZMQ_UNUSED (subscribe_to_all_);
 
     zmq_assert (pipe_);
     fq.attach (pipe_);
@@ -103,14 +104,11 @@ int zmq::xsub_t::xsend (msg_t *msg_)
         //  when there are forwarding devices involved.
         subscriptions.add (data + 1, size - 1);
         return dist.send_to_all (msg_);
-    }
-    else
-    if (size > 0 && *data == 0) {
+    } else if (size > 0 && *data == 0) {
         //  Process unsubscribe message
         if (subscriptions.rm (data + 1, size - 1))
             return dist.send_to_all (msg_);
-    }
-    else
+    } else
         //  User message sent upstream to XPUB socket
         return dist.send_to_all (msg_);
 
@@ -144,7 +142,6 @@ int zmq::xsub_t::xrecv (msg_t *msg_)
     //  stream of non-matching messages which breaks the non-blocking recv
     //  semantics.
     while (true) {
-
         //  Get a message using fair queueing algorithm.
         int rc = fq.recv (msg_);
 
@@ -183,7 +180,6 @@ bool zmq::xsub_t::xhas_in ()
     //  TODO: This can result in infinite loop in the case of continuous
     //  stream of non-matching messages.
     while (true) {
-
         //  Get a message using fair queueing algorithm.
         int rc = fq.recv (&message);
 
@@ -209,28 +205,37 @@ bool zmq::xsub_t::xhas_in ()
     }
 }
 
-zmq::blob_t zmq::xsub_t::get_credential () const
+const zmq::blob_t &zmq::xsub_t::get_credential () const
 {
     return fq.get_credential ();
 }
 
 bool zmq::xsub_t::match (msg_t *msg_)
 {
-    return subscriptions.check ((unsigned char*) msg_->data (), msg_->size ());
+    bool matching =
+      subscriptions.check ((unsigned char *) msg_->data (), msg_->size ());
+
+    return matching ^ options.invert_matching;
 }
 
-void zmq::xsub_t::send_subscription (unsigned char *data_, size_t size_,
-    void *arg_)
+void zmq::xsub_t::send_subscription (unsigned char *data_,
+                                     size_t size_,
+                                     void *arg_)
 {
-    pipe_t *pipe = (pipe_t*) arg_;
+    pipe_t *pipe = (pipe_t *) arg_;
 
-    //  Create the subsctription message.
+    //  Create the subscription message.
     msg_t msg;
     int rc = msg.init_size (size_ + 1);
     errno_assert (rc == 0);
-    unsigned char *data = (unsigned char*) msg.data ();
-    data [0] = 1;
-    memcpy (data + 1, data_, size_);
+    unsigned char *data = (unsigned char *) msg.data ();
+    data[0] = 1;
+
+    //  We explicitly allow a NULL subscription with size zero
+    if (size_) {
+        assert (data_);
+        memcpy (data + 1, data_, size_);
+    }
 
     //  Send it to the pipe.
     bool sent = pipe->write (&msg);
