@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2016 Contributors as noted in the AUTHORS file
 
     This file is part of libzmq, the ZeroMQ core engine in C++.
 
@@ -39,7 +39,7 @@ void test_system_max ()
     const int no_of_sockets = 2 * 65536;
     void *ctx = zmq_ctx_new ();
     zmq_ctx_set (ctx, ZMQ_MAX_SOCKETS, no_of_sockets);
-    std::vector <void*> sockets;
+    std::vector<void *> sockets;
 
     while (true) {
         void *socket = zmq_socket (ctx, ZMQ_PAIR);
@@ -47,7 +47,7 @@ void test_system_max ()
             break;
         sockets.push_back (socket);
     }
-    assert ((int) sockets.size () < no_of_sockets);
+    assert ((int) sockets.size () <= no_of_sockets);
 
     //  System is out of resources, further calls to zmq_socket should return NULL
     for (unsigned int i = 0; i < 10; ++i) {
@@ -56,7 +56,7 @@ void test_system_max ()
     }
     // Clean up.
     for (unsigned int i = 0; i < sockets.size (); ++i)
-        zmq_close (sockets [i]);
+        zmq_close (sockets[i]);
 
     zmq_ctx_destroy (ctx);
 }
@@ -65,7 +65,7 @@ void test_zmq_default_max ()
 {
     //  Keep allocating sockets until we hit the default limit
     void *ctx = zmq_ctx_new ();
-    std::vector<void*> sockets;
+    std::vector<void *> sockets;
 
     while (true) {
         void *socket = zmq_socket (ctx, ZMQ_PAIR);
@@ -84,7 +84,7 @@ void test_zmq_default_max ()
 
     //  Clean up
     for (unsigned int i = 0; i < sockets.size (); ++i)
-        zmq_close (sockets [i]);
+        zmq_close (sockets[i]);
 
     zmq_ctx_destroy (ctx);
 }
