@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2017 Contributors as noted in the AUTHORS file
 
     This file is part of libzmq, the ZeroMQ core engine in C++.
 
@@ -31,32 +31,36 @@
 
 int main (void)
 {
-    setup_test_environment();
+    setup_test_environment ();
+    size_t len = MAX_SOCKET_STRING;
+    char my_endpoint[MAX_SOCKET_STRING];
     void *ctx = zmq_ctx_new ();
     assert (ctx);
 
     //  Create server and bind to endpoint
     void *server = zmq_socket (ctx, ZMQ_ROUTER);
     assert (server);
-    int rc = zmq_bind (server, "tcp://127.0.0.1:5560");
+    int rc = zmq_bind (server, "tcp://127.0.0.1:*");
+    assert (rc == 0);
+    rc = zmq_getsockopt (server, ZMQ_LAST_ENDPOINT, my_endpoint, &len);
     assert (rc == 0);
 
     //  Create client and connect to server, doing a probe
     void *client = zmq_socket (ctx, ZMQ_ROUTER);
     assert (client);
-    rc = zmq_setsockopt (client, ZMQ_IDENTITY, "X", 1);
+    rc = zmq_setsockopt (client, ZMQ_ROUTING_ID, "X", 1);
     assert (rc == 0);
     int probe = 1;
     rc = zmq_setsockopt (client, ZMQ_PROBE_ROUTER, &probe, sizeof (probe));
     assert (rc == 0);
-    rc = zmq_connect (client, "tcp://localhost:5560");
+    rc = zmq_connect (client, my_endpoint);
     assert (rc == 0);
 
-    //  We expect an identity=X + empty message from client
-    unsigned char buffer [255];
+    //  We expect a routing id=X + empty message from client
+    unsigned char buffer[255];
     rc = zmq_recv (server, buffer, 255, 0);
     assert (rc == 1);
-    assert (buffer [0] ==  'X');
+    assert (buffer[0] == 'X');
     rc = zmq_recv (server, buffer, 255, 0);
     assert (rc == 0);
 
@@ -78,5 +82,5 @@ int main (void)
     rc = zmq_ctx_term (ctx);
     assert (rc == 0);
 
-    return 0 ;
+    return 0;
 }
