@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2017 Contributors as noted in the AUTHORS file
 
     This file is part of libzmq, the ZeroMQ core engine in C++.
 
@@ -31,7 +31,9 @@
 
 int main (void)
 {
-    setup_test_environment();
+    setup_test_environment ();
+    size_t len = MAX_SOCKET_STRING;
+    char my_endpoint[MAX_SOCKET_STRING];
     void *ctx = zmq_ctx_new ();
     assert (ctx);
 
@@ -49,10 +51,12 @@ int main (void)
     rc = zmq_setsockopt (req, ZMQ_RCVTIMEO, &rcvtimeo, sizeof (int));
     assert (rc == 0);
 
-    rc = zmq_connect (req, "tcp://localhost:5555");
+    rc = zmq_bind (router, "tcp://127.0.0.1:*");
+    assert (rc == 0);
+    rc = zmq_getsockopt (router, ZMQ_LAST_ENDPOINT, my_endpoint, &len);
     assert (rc == 0);
 
-    rc = zmq_bind (router, "tcp://127.0.0.1:5555");
+    rc = zmq_connect (req, my_endpoint);
     assert (rc == 0);
 
     // Send a multi-part request.
@@ -61,7 +65,7 @@ int main (void)
     zmq_msg_t msg;
     zmq_msg_init (&msg);
 
-    // Receive peer identity
+    // Receive peer routing id
     rc = zmq_msg_recv (&msg, router, 0);
     assert (rc != -1);
     assert (zmq_msg_size (&msg) > 0);
@@ -78,7 +82,7 @@ int main (void)
     // Receive request id 1
     rc = zmq_msg_recv (&msg, router, 0);
     assert (rc != -1);
-    assert (zmq_msg_size (&msg) == sizeof(uint32_t));
+    assert (zmq_msg_size (&msg) == sizeof (uint32_t));
     uint32_t req_id = *static_cast<uint32_t *> (zmq_msg_data (&msg));
     zmq_msg_t req_id_msg;
     zmq_msg_init (&req_id_msg);
@@ -93,58 +97,7 @@ int main (void)
     // Receive the rest.
     s_recv_seq (router, 0, "ABC", "DEF", SEQ_END);
 
-    // Send back a bad reply: correct req id
-    zmq_msg_copy (&msg, &peer_id_msg);
-    rc = zmq_msg_send (&msg, router, ZMQ_SNDMORE);
-    assert (rc != -1);
-    zmq_msg_copy (&msg, &req_id_msg);
-    rc = zmq_msg_send (&msg, router, 0);
-    assert (rc != -1);
-
-    // Send back a bad reply: wrong req id
-    zmq_msg_copy (&msg, &peer_id_msg);
-    rc = zmq_msg_send (&msg, router, ZMQ_SNDMORE);
-    assert (rc != -1);
     uint32_t bad_req_id = req_id + 1;
-    zmq_msg_init_data (&msg, &bad_req_id, sizeof (uint32_t), NULL, NULL);
-    rc = zmq_msg_send (&msg, router, 0);
-    assert (rc != -1);
-
-    // Send back a bad reply: correct req id, 0
-    zmq_msg_copy (&msg, &peer_id_msg);
-    rc = zmq_msg_send (&msg, router, ZMQ_SNDMORE);
-    assert (rc != -1);
-    zmq_msg_copy (&msg, &req_id_msg);
-    rc = zmq_msg_send (&msg, router, ZMQ_SNDMORE);
-    assert (rc != -1);
-    s_send_seq (router, 0, SEQ_END);
-
-    // Send back a bad reply: correct req id, garbage
-    zmq_msg_copy (&msg, &peer_id_msg);
-    rc = zmq_msg_send (&msg, router, ZMQ_SNDMORE);
-    assert (rc != -1);
-    zmq_msg_copy (&msg, &req_id_msg);
-    rc = zmq_msg_send (&msg, router, ZMQ_SNDMORE);
-    assert (rc != -1);
-    s_send_seq (router, "FOO", SEQ_END);
-
-    // Send back a bad reply: wrong req id, 0
-    zmq_msg_copy (&msg, &peer_id_msg);
-    rc = zmq_msg_send (&msg, router, ZMQ_SNDMORE);
-    assert (rc != -1);
-    zmq_msg_init_data (&msg, &bad_req_id, sizeof (uint32_t), NULL, NULL);
-    rc = zmq_msg_send (&msg, router, ZMQ_SNDMORE);
-    assert (rc != -1);
-    s_send_seq (router, 0, SEQ_END);
-
-    // Send back a bad reply: correct req id, garbage, data
-    zmq_msg_copy (&msg, &peer_id_msg);
-    rc = zmq_msg_send (&msg, router, ZMQ_SNDMORE);
-    assert (rc != -1);
-    zmq_msg_copy (&msg, &req_id_msg);
-    rc = zmq_msg_send (&msg, router, ZMQ_SNDMORE);
-    assert (rc != -1);
-    s_send_seq (router, "FOO", "DATA", SEQ_END);
 
     // Send back a bad reply: wrong req id, 0, data
     zmq_msg_copy (&msg, &peer_id_msg);
@@ -155,7 +108,7 @@ int main (void)
     assert (rc != -1);
     s_send_seq (router, 0, "DATA", SEQ_END);
 
-    // Send back a good reply.
+    // Send back a good reply: good req id, 0, data
     zmq_msg_copy (&msg, &peer_id_msg);
     rc = zmq_msg_send (&msg, router, ZMQ_SNDMORE);
     assert (rc != -1);
@@ -164,7 +117,7 @@ int main (void)
     assert (rc != -1);
     s_send_seq (router, 0, "GHI", SEQ_END);
 
-    // Receive reply. If any of the other messages got through, we wouldn't see
+    // Receive reply. If bad reply got through, we wouldn't see
     // this particular data.
     s_recv_seq (req, "GHI", SEQ_END);
 
