@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2017 Contributors as noted in the AUTHORS file
 
     This file is part of libzmq, the ZeroMQ core engine in C++.
 
@@ -30,7 +30,7 @@
 #include "testutil.hpp"
 
 const char *bind_address = 0;
-const char *connect_address = 0;
+char connect_address[MAX_SOCKET_STRING];
 
 void test_fair_queue_in (void *ctx)
 {
@@ -43,41 +43,47 @@ void test_fair_queue_in (void *ctx)
 
     rc = zmq_bind (receiver, bind_address);
     assert (rc == 0);
+    size_t len = MAX_SOCKET_STRING;
+    rc = zmq_getsockopt (receiver, ZMQ_LAST_ENDPOINT, connect_address, &len);
+    assert (rc == 0);
 
     const size_t services = 5;
-    void *senders [services];
+    void *senders[services];
     for (size_t peer = 0; peer < services; ++peer) {
-        senders [peer] = zmq_socket (ctx, ZMQ_DEALER);
-        assert (senders [peer]);
+        senders[peer] = zmq_socket (ctx, ZMQ_DEALER);
+        assert (senders[peer]);
 
-        rc = zmq_setsockopt (senders [peer], ZMQ_RCVTIMEO, &timeout, sizeof (int));
+        rc =
+          zmq_setsockopt (senders[peer], ZMQ_RCVTIMEO, &timeout, sizeof (int));
         assert (rc == 0);
 
-        char *str = strdup("A");
-        str [0] += peer;
-        rc = zmq_setsockopt (senders [peer], ZMQ_IDENTITY, str, 2);
+        char *str = strdup ("A");
+        str[0] += peer;
+        rc = zmq_setsockopt (senders[peer], ZMQ_ROUTING_ID, str, 2);
         assert (rc == 0);
         free (str);
 
-        rc = zmq_connect (senders [peer], connect_address);
+        rc = zmq_connect (senders[peer], connect_address);
         assert (rc == 0);
     }
 
+    msleep (SETTLE_TIME);
+
     zmq_msg_t msg;
     rc = zmq_msg_init (&msg);
     assert (rc == 0);
 
-    s_send_seq (senders [0], "M", SEQ_END);
+    s_send_seq (senders[0], "M", SEQ_END);
     s_recv_seq (receiver, "A", "M", SEQ_END);
 
-    s_send_seq (senders [0], "M", SEQ_END);
+    s_send_seq (senders[0], "M", SEQ_END);
     s_recv_seq (receiver, "A", "M", SEQ_END);
 
     int sum = 0;
 
     // send N requests
     for (size_t peer = 0; peer < services; ++peer) {
-        s_send_seq (senders [peer], "M", SEQ_END);
+        s_send_seq (senders[peer], "M", SEQ_END);
         sum += 'A' + peer;
     }
 
@@ -87,8 +93,8 @@ void test_fair_queue_in (void *ctx)
     for (size_t peer = 0; peer < services; ++peer) {
         rc = zmq_msg_recv (&msg, receiver, 0);
         assert (rc == 2);
-        const char *id = (const char *)zmq_msg_data (&msg);
-        sum -= id [0];
+        const char *id = (const char *) zmq_msg_data (&msg);
+        sum -= id[0];
 
         s_recv_seq (receiver, "M", SEQ_END);
     }
@@ -101,11 +107,10 @@ void test_fair_queue_in (void *ctx)
     close_zero_linger (receiver);
 
     for (size_t peer = 0; peer < services; ++peer)
-        close_zero_linger (senders [peer]);
+        close_zero_linger (senders[peer]);
 
     // Wait for disconnects.
-    rc = zmq_poll (0, 0, 100);
-    assert (rc == 0);
+    msleep (SETTLE_TIME);
 }
 
 void test_destroy_queue_on_disconnect (void *ctx)
@@ -114,24 +119,27 @@ void test_destroy_queue_on_disconnect (void *ctx)
     assert (A);
 
     int enabled = 1;
-    int rc = zmq_setsockopt (A, ZMQ_ROUTER_MANDATORY, &enabled, sizeof (enabled));
+    int rc =
+      zmq_setsockopt (A, ZMQ_ROUTER_MANDATORY, &enabled, sizeof (enabled));
     assert (rc == 0);
 
     rc = zmq_bind (A, bind_address);
     assert (rc == 0);
+    size_t len = MAX_SOCKET_STRING;
+    rc = zmq_getsockopt (A, ZMQ_LAST_ENDPOINT, connect_address, &len);
+    assert (rc == 0);
 
     void *B = zmq_socket (ctx, ZMQ_DEALER);
     assert (B);
 
-    rc = zmq_setsockopt (B, ZMQ_IDENTITY, "B", 2);
+    rc = zmq_setsockopt (B, ZMQ_ROUTING_ID, "B", 2);
     assert (rc == 0);
 
     rc = zmq_connect (B, connect_address);
     assert (rc == 0);
 
     // Wait for connection.
-    rc = zmq_poll (0, 0, 100);
-    assert (rc == 0);
+    msleep (SETTLE_TIME);
 
     // Send a message in both directions
     s_send_seq (A, "B", "ABC", SEQ_END);
@@ -141,7 +149,7 @@ void test_destroy_queue_on_disconnect (void *ctx)
     assert (rc == 0);
 
     // Disconnect may take time and need command processing.
-    zmq_pollitem_t poller [2] = { { A, 0, 0, 0 }, { B, 0, 0, 0 } };
+    zmq_pollitem_t poller[2] = {{A, 0, 0, 0}, {B, 0, 0, 0}};
     rc = zmq_poll (poller, 2, 100);
     assert (rc == 0);
     rc = zmq_poll (poller, 2, 100);
@@ -178,23 +186,20 @@ void test_destroy_queue_on_disconnect (void *ctx)
     close_zero_linger (B);
 
     // Wait for disconnects.
-    rc = zmq_poll (0, 0, 100);
-    assert (rc == 0);
+    msleep (SETTLE_TIME);
 }
 
 
 int main (void)
 {
-    setup_test_environment();
+    setup_test_environment ();
     void *ctx = zmq_ctx_new ();
     assert (ctx);
 
-    const char *binds [] = { "inproc://a", "tcp://127.0.0.1:5555" };
-    const char *connects [] = { "inproc://a", "tcp://localhost:5555" };
+    const char *binds[] = {"inproc://a", "tcp://127.0.0.1:*"};
 
     for (int transport = 0; transport < 2; ++transport) {
-        bind_address = binds [transport];
-        connect_address = connects [transport];
+        bind_address = binds[transport];
 
         // SHALL receive incoming messages from its peers using a fair-queuing
         // strategy.
@@ -210,5 +215,5 @@ int main (void)
     int rc = zmq_ctx_term (ctx);
     assert (rc == 0);
 
-    return 0 ;
+    return 0;
 }
